# command_status.py
#
# Copyright (C) 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys, re, getopt
import the
from deployment import Deployment
from walker.status_walker import StatusWalker


class CommandStatus:


	def __init__( self ):
		self.repo = None


	def print_help( self ):
		print "Usage: " + the.program.name + " status [--repo=REPO]"
		print
		#      01234567890123456789012345678901234567890123456789012345678901234567890123456789
		print "Show the state of files in the local repository (including modifications and"
		print "conflicts)."
		print
		print "This lists files in your home directory that have been modified and which differ"
		print "from those in the local repository.  During an update that resulted in conflicts"
		print "this also lists files which have conflicts in the local local respository, or"
		print "which can not be deployed due to conflicts with your home directory."
		print
		print "For help with adding modified files to the repository, type:"
		print "    " + the.program.name + " add --help"
		print
		print "Options:"
		print "  -r, --repo=REPO  select the repo to check-out or create (defaults to 'home')"
		print "  -v, --verbose    display information about what is being done"
		print "      --help       display help and exit"
		exit( 0 )


	def parse_command_line( self ):
		opts, args = getopt.gnu_getopt(
			sys.argv[ 1: ], "r:v",
			[ "repo=", "verbose", "help" ] )
		for opt, optarg in opts:
			if opt in [ '--repo', '-r' ]:
				if not re.match( '^[-a-zA-z0-9.]+$', optarg ):
					raise the.program.FatalError(
						'invalid repository name: ' + optarg )
				self.repo = optarg
			elif opt in [ '--verbose', '-v' ]:
				the.verbose = True
			elif opt == "--help":
				self.print_help()
		
		# discard first argument (the command)
		args.pop( 0 )

		# remaining arguments
		if len( args ):
			raise the.program.UsageError( 'too many arguments' )


	def run( self ):

		# set up repo and check it exists
		the.set_repo( self.repo )
		the.repo.check_dir_exists()

		message = ''

		# initialise deployment and check if it's ongoing
		deployment = Deployment()
		if deployment.is_ongoing():

			# check for conflicts in repo
			files = the.repo.vcs.get_conflicts()
			if files:
				message += 'Conflicts in %s:\n  %s\n' % \
						   ( the.repo.name, '\n  '.join( files ) )

			# get deployment conflicts
			conflicts = deployment.get_conflicts()
			if conflicts:
				message += 'Deployment conflicts:\n  %s\n' % \
						   '\n  '.join( conflicts )

		else:

			# check status
			walker = StatusWalker()
			walker.walk()
			if walker.modified:
				message += 'Modified:\n  %s\n' % \
						   '\n  '.join( walker.modified )
			if walker.missing:
				message += 'Missing:\n  %s\n' % \
						   '\n  '.join( walker.missing )
			if walker.changed:
				message += 'Type changed:\n  %s\n' % \
						   '\n  '.join( walker.changed )

		# show status
		if message: print message.rstrip()
